@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package AnyEvent::InfluxDB;
 our $AUTHORITY = 'cpan:AJGB';
-$AnyEvent::InfluxDB::VERSION = '0.13';
+$AnyEvent::InfluxDB::VERSION = '1.0.2.0';
 use AnyEvent;
 use AnyEvent::HTTP;
 use URI;
@@ -13,7 +13,7 @@ use List::MoreUtils qw(zip);
 use URI::Encode::XS qw( uri_encode );
 use Moo;
 
-has [qw( ssl_options username password on_request )] => (
+has [qw( ssl_options username password jwt on_request )] => (
     is => 'ro',
     predicate => 1,
 );
@@ -93,6 +93,11 @@ sub _http_request {
             'user-agent' => "AnyEvent-InfluxDB/0.13",
         }
     );
+
+    if ($self->has_jwt) {
+        $args{headers}->{Authorization} = 'Bearer '. $self->jwt;
+    }
+
     if ( $method eq 'POST' ) {
         if ( defined $post_data ) {
             $args{'body'} = $post_data;
@@ -143,6 +148,199 @@ sub ping {
 }
 
 
+sub _to_line {
+    my $data = shift;
+
+    my $t = $data->{tags} || {};
+    my $f = $data->{fields} || {};
+
+    return $data->{measurement}
+        .(
+            $t ?
+                    ','.
+                    join(',',
+                        map {
+                            join('=', $_, $t->{$_})
+                        } sort { $a cmp $b } keys %$t
+                    )
+                :
+                ''
+        )
+        . ' '
+        .(
+            join(',',
+                map {
+                    join('=', $_, $f->{$_})
+                } keys %$f
+            )
+        )
+        .(
+            $data->{time} ?
+                ' '. $data->{time}
+                :
+                ''
+        );
+}
+
+sub write {
+    my ($self, %args) = @_;
+
+    my $data = ref $args{data} eq 'ARRAY' ?
+        join("\n", map { ref $_ eq 'HASH' ? _to_line($_) : $_ } @{ $args{data} })
+        :
+        ref $args{data} eq 'HASH' ? _to_line($args{data}) : $args{data};
+
+    my $url = $self->_make_url('/write', {
+        db => $args{database},
+        (
+            $args{consistency} ?
+                ( consistency => $args{consistency} )
+                :
+                ()
+        ),
+        (
+            $args{rp} ?
+                ( rp => $args{rp} )
+                :
+                ()
+        ),
+        (
+            $args{precision} ?
+                ( precision => $args{precision} )
+                :
+                ()
+        ),
+        (
+            $args{one} ?
+                ( one => $args{one} )
+                :
+                ()
+        )
+    });
+
+    $self->_http_request( POST => $url, $data,
+        sub {
+            my ($body, $headers) = @_;
+
+            if ( $headers->{Status} eq '204' ) {
+                $args{on_success}->();
+            } else {
+                $args{on_error}->( $body );
+            }
+        }
+    );
+}
+
+
+sub select {
+    my ($self, %args) = @_;
+
+    my $method = 'GET';
+    my $q;
+    if ( exists $args{q} ) {
+        $q = $args{q};
+        if ( $q =~ /\s+INTO\s+/i ) {
+            $method = 'POST';
+        }
+    } else {
+        $q = 'SELECT '. $args{fields};
+
+        if ( my $into = $args{into} ) {
+            $q .= ' INTO '. $into;
+            $method = 'POST';
+        }
+
+        $q .= ' FROM '. $args{measurement};
+
+        if ( my $cond = $args{where} ) {
+            $q .= ' WHERE '. $cond;
+        }
+
+        if ( my $group = $args{group_by} ) {
+            $q .= ' GROUP BY '. $group;
+
+            if ( my $fill = $args{fill} ) {
+                $q .= ' fill('. $fill .')';
+            }
+        }
+
+        if ( my $order_by = $args{order_by} ) {
+            $q .= ' ORDER BY '. $order_by;
+        }
+
+        if ( my $limit = $args{limit} ) {
+            $q .= ' LIMIT '. $limit;
+
+            if ( my $offset = $args{offset} ) {
+                $q .= ' OFFSET '. $offset;
+            }
+        }
+
+        if ( my $slimit = $args{slimit} ) {
+            $q .= ' SLIMIT '. $slimit;
+
+            if ( my $soffset = $args{soffset} ) {
+                $q .= ' SOFFSET '. $soffset;
+            }
+        }
+    }
+
+    my $url = $self->_make_url('/query', {
+        db => $args{database},
+        q => $q,
+        (
+            $args{rp} ?
+                ( rp => $args{rp} )
+                :
+                ()
+        ),
+        (
+            $args{epoch} ?
+                ( epoch => $args{epoch} )
+                :
+                ()
+        ),
+        (
+            $args{chunk_size} ?
+                ( chunk_size => $args{chunk_size} )
+                :
+                ()
+        ),
+    });
+
+    $self->_http_request( $method => $url,
+        sub {
+            my ($body, $headers) = @_;
+
+            if ( $headers->{Status} eq '200' ) {
+                my $data = decode_json($body);
+                my $series = [
+                    map {
+                        my $res = $_;
+
+                        my $cols = $res->{columns};
+                        my $values = $res->{values};
+
+                        +{
+                            name => $res->{name},
+                            values => [
+                                map {
+                                    +{
+                                        zip(@$cols, @$_)
+                                    }
+                                } @{ $values || [] }
+                            ]
+                        }
+                    } @{ $data->{results}->[0]->{series} || [] }
+                ];
+                $args{on_success}->($series);
+            } else {
+                $args{on_error}->( $body );
+            }
+        }
+    );
+}
+
 
 sub create_database {
     my ($self, %args) = @_;
@@ -151,11 +349,7 @@ sub create_database {
     if ( exists $args{q} ) {
         $q = $args{q};
     } else {
-        $q = 'CREATE DATABASE '.
-            (
-                $args{if_not_exists} ? 'IF NOT EXISTS ' : ''
-            )
-            .$args{database};
+        $q = 'CREATE DATABASE '. $args{database};
     }
 
     my $url = $self->_make_url('/query', {
@@ -183,11 +377,7 @@ sub drop_database {
     if ( exists $args{q} ) {
         $q = $args{q};
     } else {
-        $q = 'DROP DATABASE '
-            .
-            (
-                $args{if_exists} ? 'IF EXISTS ' : ''
-            ) . $args{database};
+        $q = 'DROP DATABASE '. $args{database};
     }
 
     my $url = $self->_make_url('/query', {
@@ -198,7 +388,7 @@ sub drop_database {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -238,7 +428,7 @@ sub drop_series {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -275,7 +465,7 @@ sub delete_series {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -304,7 +494,7 @@ sub drop_measurement {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -399,7 +589,7 @@ sub drop_shard {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -460,7 +650,7 @@ sub kill_query {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -495,7 +685,7 @@ sub create_retention_policy {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -529,7 +719,7 @@ sub alter_retention_policy {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -557,7 +747,7 @@ sub drop_retention_policy {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -895,7 +1085,10 @@ sub show_field_keys {
                     my $values = $res->{values};
                     $field_keys->{ $res->{name } } = [
                         map {
-                            @$_
+                            +{
+                                name => $_->[0],
+                                type => $_->[1],
+                            }
                         } @{ $values || [] }
                     ];
                 }
@@ -929,7 +1122,7 @@ sub create_user {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -958,7 +1151,7 @@ sub set_user_password {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -1025,7 +1218,7 @@ sub grant_privileges {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -1100,7 +1293,7 @@ sub revoke_privileges {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -1129,7 +1322,7 @@ sub drop_user {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -1171,7 +1364,7 @@ sub create_continuous_query {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -1199,7 +1392,7 @@ sub drop_continuous_query {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -1244,218 +1437,6 @@ sub show_continuous_queries {
 }
 
 
-
-sub _to_line {
-    my $data = shift;
-
-    my $t = $data->{tags} || {};
-    my $f = $data->{fields} || {};
-
-    return $data->{measurement}
-        .(
-            $t ?
-                    ','.
-                    join(',',
-                        map {
-                            join('=', $_, $t->{$_})
-                        } sort { $a cmp $b } keys %$t
-                    )
-                :
-                ''
-        )
-        . ' '
-        .(
-            join(',',
-                map {
-                    join('=', $_, $f->{$_})
-                } keys %$f
-            )
-        )
-        .(
-            $data->{time} ?
-                ' '. $data->{time}
-                :
-                ''
-        );
-}
-
-sub write {
-    my ($self, %args) = @_;
-
-    my $data = ref $args{data} eq 'ARRAY' ?
-        join("\n", map { ref $_ eq 'HASH' ? _to_line($_) : $_ } @{ $args{data} })
-        :
-        ref $args{data} eq 'HASH' ? _to_line($args{data}) : $args{data};
-
-    my $url = $self->_make_url('/write', {
-        db => $args{database},
-        (
-            $args{consistency} ?
-                ( consistency => $args{consistency} )
-                :
-                ()
-        ),
-        (
-            $args{rp} ?
-                ( rp => $args{rp} )
-                :
-                ()
-        ),
-        (
-            $args{precision} ?
-                ( precision => $args{precision} )
-                :
-                ()
-        ),
-        (
-            $args{one} ?
-                ( one => $args{one} )
-                :
-                ()
-        )
-    });
-
-    $self->_http_request( POST => $url, $data,
-        sub {
-            my ($body, $headers) = @_;
-
-            if ( $headers->{Status} eq '204' ) {
-                $args{on_success}->();
-            } else {
-                $args{on_error}->( $body );
-            }
-        }
-    );
-}
-
-
-sub select {
-    my ($self, %args) = @_;
-
-    my $method = 'GET';
-    my $q;
-    if ( exists $args{q} ) {
-        $q = $args{q};
-        if ( $q =~ /\s+INTO\s+/i ) {
-            $method = 'POST';
-        }
-    } else {
-        $q = 'SELECT '. $args{fields};
-
-        if ( my $into = $args{into} ) {
-            $q .= ' INTO '. $into;
-            $method = 'POST';
-        }
-
-        $q .= ' FROM '. $args{measurement};
-
-        if ( my $cond = $args{where} ) {
-            $q .= ' WHERE '. $cond;
-        }
-
-        if ( my $group = $args{group_by} ) {
-            $q .= ' GROUP BY '. $group;
-
-            if ( my $fill = $args{fill} ) {
-                $q .= ' fill('. $fill .')';
-            }
-        }
-
-        if ( my $order_by = $args{order_by} ) {
-            $q .= ' ORDER BY '. $order_by;
-        }
-
-        if ( my $limit = $args{limit} ) {
-            $q .= ' LIMIT '. $limit;
-
-            if ( my $offset = $args{offset} ) {
-                $q .= ' OFFSET '. $offset;
-            }
-        }
-
-        if ( my $slimit = $args{slimit} ) {
-            $q .= ' SLIMIT '. $slimit;
-
-            if ( my $soffset = $args{soffset} ) {
-                $q .= ' SOFFSET '. $soffset;
-            }
-        }
-    }
-
-    my $url = $self->_make_url('/query', {
-        db => $args{database},
-        q => $q,
-        (
-            $args{rp} ?
-                ( rp => $args{rp} )
-                :
-                ()
-        ),
-        (
-            $args{epoch} ?
-                ( epoch => $args{epoch} )
-                :
-                ()
-        ),
-        (
-            $args{chunk_size} ?
-                ( chunk_size => $args{chunk_size} )
-                :
-                ()
-        ),
-    });
-
-    $self->_http_request( $method => $url,
-        sub {
-            my ($body, $headers) = @_;
-
-            if ( $headers->{Status} eq '200' ) {
-                my $data = decode_json($body);
-                my $series = [
-                    map {
-                        my $res = $_;
-
-                        my $cols = $res->{columns};
-                        my $values = $res->{values};
-
-                        +{
-                            name => $res->{name},
-                            values => [
-                                map {
-                                    +{
-                                        zip(@$cols, @$_)
-                                    }
-                                } @{ $values || [] }
-                            ]
-                        }
-                    } @{ $data->{results}->[0]->{series} || [] }
-                ];
-                $args{on_success}->($series);
-            } else {
-                $args{on_error}->( $body );
-            }
-        }
-    );
-}
-
-
-sub query {
-    my ($self, %args) = @_;
-
-    my $url = $self->_server_uri->clone;
-    $url->path('/query');
-    $url->query_form_hash( $args{query} );
-
-    my $method = $args{method} || 'GET';
-
-    $self->_http_request( $method => $url,
-        sub {
-            $args{on_response}->(@_);
-        }
-    );
-}
-
-
 sub create_subscription {
     my ($self, %args) = @_;
 
@@ -1482,7 +1463,7 @@ sub create_subscription {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -1545,7 +1526,7 @@ sub drop_subscription {
         sub {
             my ($body, $headers) = @_;
 
-            if ( $headers->{Status} eq '200' && $body eq '{"results":[{}]}' ) {
+            if ( $headers->{Status} eq '200' ) {
                 $args{on_success}->();
             } else {
                 $args{on_error}->( $body );
@@ -1555,6 +1536,22 @@ sub drop_subscription {
 }
 
 
+sub query {
+    my ($self, %args) = @_;
+
+    my $url = $self->_server_uri->clone;
+    $url->path('/query');
+    $url->query_form_hash( $args{query} );
+
+    my $method = $args{method} || 'GET';
+
+    $self->_http_request( $method => $url,
+        sub {
+            $args{on_response}->(@_);
+        }
+    );
+}
+
 
 1;
 
@@ -1570,7 +1567,7 @@ AnyEvent::InfluxDB - An asynchronous library for InfluxDB time-series database
 
 =head1 VERSION
 
-version 0.13
+version 1.0.2.0
 
 =head1 SYNOPSIS
 
@@ -1641,7 +1638,7 @@ version 0.13
 
 Asynchronous client library for InfluxDB time-series database L<https://influxdb.com>.
 
-This version is meant to be used with InfluxDB v0.13 or newer.
+This version is meant to be used with InfluxDB v1.0.0 or newer.
 
 =head1 METHODS
 
@@ -1649,8 +1646,13 @@ This version is meant to be used with InfluxDB v0.13 or newer.
 
     my $db = AnyEvent::InfluxDB->new(
         server => 'http://localhost:8086',
+
+        # authenticate using Basic credentials
         username => 'admin',
         password => 'password',
+
+        # or use JWT token
+        jwt => 'JWT_TOKEN_BLOB'
     );
 
 Returns object representing given InfluDB C<server> connected using optionally
@@ -1663,9 +1665,7 @@ host certificate is performed. This can be changed by setting C<ssl_options>
 parameter with any options accepted by L<AnyEvent::TLS>.
 
     my $db = AnyEvent::InfluxDB->new(
-        server => 'https://localhost:8086',
-        username => 'admin',
-        password => 'password',
+        ...
         ssl_options => {
             verify => 1,
             verify_peername => 'https',
@@ -1677,6 +1677,7 @@ As an debugging aid the C<on_request> code reference may also be provided. It wi
 be executed before each request with the method name, url and POST data if set.
 
     my $db = AnyEvent::InfluxDB->new(
+        ...
         on_request => sub {
             my ($method, $url, $post_data) = @_;
             print "$method $url\n";
@@ -1684,7 +1685,7 @@ be executed before each request with the method name, url and POST data if set.
         }
     );
 
-=for Pod::Coverage has_on_request has_password has_ssl_options has_username on_request password server ssl_options username
+=for Pod::Coverage has_jwt jwt has_on_request has_password has_ssl_options has_username on_request password server ssl_options username
 
 =head2 ping
 
@@ -1708,6 +1709,136 @@ with the value of C<X-Influxdb-Version> response header as argument,
 otherwise executes the required C<on_error> code reference with the value of
 C<Reason> response header as argument.
 
+=head2 Managing Data
+
+=head3 write
+
+    $cv = AE::cv;
+    $db->write(
+        database => 'mydb',
+        precision => 's',
+        rp => 'last_day',
+        consistency => 'quorum',
+
+        data => [
+            # line protocol formatted
+            'cpu_load,host=server02,region=eu-east sensor="top",value=0.64 1456097956',
+
+            # or as a hash
+            {
+                measurement => 'cpu_load',
+                tags => {
+                    host => 'server02',
+                    region => 'eu-east',
+                },
+                fields => {
+                    value => '0.64',
+                    sensor => q{"top"},
+                },
+                time => time()
+            }
+        ],
+
+        on_success => $cv,
+        on_error => sub {
+            $cv->croak("Failed to write data: @_");
+        }
+    );
+    $cv->recv;
+
+Writes time-series data C<data> to database C<database> with optional parameters:
+retention policy C<rp>, time precision C<precision> and consistency C<consistency>.
+
+The required C<on_success> code reference is executed if request was successful,
+otherwise executes the required C<on_error> code reference.
+
+The C<data> can be specified as single scalar value or hash reference with
+required keys C<measurement> and C<fields> and optional C<tags> and C<time>.
+Both can be also mixed and matched within an array reference.
+
+Scalar values are expected to be formatted using InfluxDB line protocol.
+
+All special characters need to be escaped. In that case you might want to use
+L<InfluxDB::LineProtocol>:
+
+    use InfluxDB::LineProtocol qw(dataline);
+
+    ...
+    $db->write(
+        database => 'mydb',
+        precision => 'n',
+
+        data => [
+            dataline('CPU Load', 0.64, { "Region of the World" => "Eastern Europe", codename => "eu-east" }, 1437868012260500137)
+
+            # which translates to
+            'CPU\ Load,Region\ of\ the\ World=Eastern\ Europe,codename=eu-east value=0.64 1437868012260500137',
+        ],
+        ...
+    );
+
+=head2 Querying Data
+
+=head3 select
+
+    $cv = AE::cv;
+    $db->select(
+        database => 'mydb',
+
+        # return time in Unix epoch format
+        epoch => "s",
+
+        # raw query
+        q => "SELECT count(value) FROM cpu_load"
+            ." WHERE region = 'eu-east' AND time > now() - 14d"
+            ." GROUP BY time(1d) fill(none)"
+            ." ORDER BY time DESC"
+            ." LIMIT 10 OFFSET 3",
+
+        # or query created from arguments
+        fields => 'count(value)',
+        measurement => 'cpu_load',
+        where => "region = 'eu-east' AND time > now() - 14d",
+
+        group_by => 'time(1d)',
+        fill => 'none',
+
+        order_by => 'time DESC',
+
+        limit => 10,
+        offset => 3,
+
+        # downsample result to another database, retention policy and measurement
+        into => 'otherdb."default".cpu_load_per5m',
+
+        # callbacks
+        on_success => $cv,
+        on_error => sub {
+            $cv->croak("Failed to select data: @_");
+        }
+    );
+    my $results = $cv->recv;
+    for my $row ( @{ $results } ) {
+        print "Measurement: $row->{name}\n";
+        print "Values:\n";
+        for my $value ( @{ $row->{values} || [] } ) {
+            print " * $_ = $value->{$_}\n" for keys %{ $value || {} };
+        }
+    }
+
+Executes an select query on database C<database> created from provided arguments
+measurement C<measurement>, fields to select C<fields>, optional C<where>
+clause, grouped by C<group_by> and empty values filled with C<fill>, ordered by
+C<order_by> with number of results limited to C<limit> with offset C<offset>.
+To limit number of returned series use C<slimit> with offset C<soffset>.
+If C<into> parameter is provided the result of the query will be copied to specified
+measurement.
+If C<epoch> is provided the returned C<time> value will in Unix epoch format.
+Optional C<chunk_size> can be provided to override the default value of 10,000 datapoints.
+
+The required C<on_success> code reference is executed if request was successful,
+otherwise executes the required C<on_error> code reference.
+
 =head2 Database Management
 
 =head3 create_database
@@ -1748,11 +1879,10 @@ otherwise executes the required C<on_error> code reference.
     $cv = AE::cv;
     $db->drop_database(
         # raw query
-        q => "DROP DATABASE IF EXISTS mydb",
+        q => "DROP DATABASE mydb",
 
         # or query created from arguments
         database => "mydb",
-        if_exists => 1,
 
         # callbacks
         on_success => $cv,
@@ -1762,9 +1892,7 @@ otherwise executes the required C<on_error> code reference.
     );
     $cv->recv;
 
-Drops database specified by C<database> argument. The optional parameter
-C<if_exists> if set to true value, allows to avoid error if the database
-does not exists.
+Drops database specified by C<database> argument.
 
 The required C<on_success> code reference is executed if request was successful,
 otherwise executes the required C<on_error> code reference.
@@ -1892,10 +2020,8 @@ otherwise executes the required C<on_error> code reference.
         print "Expiry Time: $sg->{expiry_time}\n";
     }
 
-Returns a list of hash references with keys C<name>, C<duration>, C<replicaN>
-and C<default> for each replication policy defined on database C<database>.
-
-Returns a hash reference with database name as keys and their shards as values.
+Returns a list of hash references with keys C<id>, C<database>, C<retention_policy>,
+C<start_time>, C<end_time> and C<expiry_time> for each shard groups.
 
 The required C<on_success> code reference is executed if request was successful,
 otherwise executes the required C<on_error> code reference.
@@ -2280,11 +2406,15 @@ otherwise executes the required C<on_error> code reference.
     my $field_keys = $cv->recv;
     for my $measurement ( sort keys %{ $field_keys } ) {
         print "Measurement: $measurement\n";
-        print " * $_\n" for @{ $field_keys->{$measurement} };
+        for my $field ( @{ $field_keys->{$measurement} } ) {
+            print "  Key:  $field->{key}\n";
+            print "  Type: $field->{type}\n";
+        }
     }
 
-Returns a hash reference with measurements as keys and their field keys
-as values from database C<database> and optional measurement C<measurement>.
+Returns a hash reference with measurements as keys and their field keys names
+and type as values from database C<database> and optional measurement
+C<measurement>.
 
 The required C<on_success> code reference is executed if request was successful,
 otherwise executes the required C<on_error> code reference.
@@ -2566,154 +2696,6 @@ continuous query defined on database C<database>.
 The required C<on_success> code reference is executed if request was successful,
 otherwise executes the required C<on_error> code reference.
 
-=head2 Managing Data
-
-=head3 write
-
-    $cv = AE::cv;
-    $db->write(
-        database => 'mydb',
-        precision => 's',
-        rp => 'last_day',
-        consistency => 'quorum',
-
-        data => [
-            # line protocol formatted
-            'cpu_load,host=server02,region=eu-east sensor="top",value=0.64 1456097956',
-
-            # or as a hash
-            {
-                measurement => 'cpu_load',
-                tags => {
-                    host => 'server02',
-                    region => 'eu-east',
-                },
-                fields => {
-                    value => '0.64',
-                    sensor => q{"top"},
-                },
-                time => time()
-            }
-        ],
-
-        on_success => $cv,
-        on_error => sub {
-            $cv->croak("Failed to write data: @_");
-        }
-    );
-    $cv->recv;
-
-Writes time-series data C<data> to database C<database> with optional parameters:
-retention policy C<rp>, time precision C<precision> and consistency C<consistency>.
-
-The required C<on_success> code reference is executed if request was successful,
-otherwise executes the required C<on_error> code reference.
-
-The C<data> can be specified as single scalar value or hash reference with
-required keys C<measurement> and C<fields> and optional C<tags> and C<time>.
-Both can be also mixed and matched within an array reference.
-
-Scalar values are expected to be formatted using InfluxDB line protocol.
-
-All special characters need to be escaped. In that case you might want to use
-L<InfluxDB::LineProtocol>:
-
-    use InfluxDB::LineProtocol qw(dataline);
-
-    ...
-    $db->write(
-        database => 'mydb',
-        precision => 'n',
-
-        data => [
-            dataline('CPU Load', 0.64, { "Region of the World" => "Eastern Europe", codename => "eu-east" }, 1437868012260500137)
-
-            # which translates to
-            'CPU\ Load,Region\ of\ the\ World=Eastern\ Europe,codename=eu-east value=0.64 1437868012260500137',
-        ],
-        ...
-    );
-
-=head2 Querying Data
-
-=head3 select
-
-    $cv = AE::cv;
-    $db->select(
-        database => 'mydb',
-
-        # return time in Unix epoch format
-        epoch => "s",
-
-        # raw query
-        q => "SELECT count(value) FROM cpu_load"
-            ." WHERE region = 'eu-east' AND time > now() - 14d"
-            ." GROUP BY time(1d) fill(none)"
-            ." ORDER BY time DESC"
-            ." LIMIT 10 OFFSET 3",
-
-        # or query created from arguments
-        fields => 'count(value)',
-        measurement => 'cpu_load',
-        where => "region = 'eu-east' AND time > now() - 14d",
-
-        group_by => 'time(1d)',
-        fill => 'none',
-
-        order_by => 'time DESC',
-
-        limit => 10,
-        offset => 3,
-
-        # downsample result to another database, retention policy and measurement
-        into => 'otherdb."default".cpu_load_per5m',
-
-        # callbacks
-        on_success => $cv,
-        on_error => sub {
-            $cv->croak("Failed to select data: @_");
-        }
-    );
-    my $results = $cv->recv;
-    for my $row ( @{ $results } ) {
-        print "Measurement: $row->{name}\n";
-        print "Values:\n";
-        for my $value ( @{ $row->{values} || [] } ) {
-            print " * $_ = $value->{$_}\n" for keys %{ $value || {} };
-        }
-    }
-
-Executes an select query on database C<database> created from provided arguments
-measurement C<measurement>, fields to select C<fields>, optional C<where>
-clause, grouped by C<group_by> and empty values filled with C<fill>, ordered by
-C<order_by> with number of results limited to C<limit> with offset C<offset>.
-To limit number of returned series use C<slimit> with offset C<soffset>.
-If C<into> parameter is provided the result of the query will be copied to specified
-measurement.
-If C<epoch> is provided the returned C<time> value will in Unix epoch format.
-Optional C<chunk_size> can be provided to override the default value of 10,000 datapoints.
-
-The required C<on_success> code reference is executed if request was successful,
-otherwise executes the required C<on_error> code reference.
-
-=head3 query
-
-    $cv = AE::cv;
-    $db->query(
-        method => 'GET',
-        query => {
-            db => 'mydb',
-            q => 'SELECT * FROM cpu_load',
-        },
-        on_response => $cv,
-    );
-    my ($response_data, $response_headers) = $cv->recv;
-
-Executes an arbitrary query using provided in C<query> arguments.
-
-The required C<on_response> code reference is executed with the raw response
-data and headers as parameters.
-
 =head2 Kapacitor integration
 
 Subscriptions tell InfluxDB to send all the data it receives to Kapacitor.
@@ -2803,6 +2785,26 @@ C<rp>.
 The required C<on_success> code reference is executed if request was successful,
 otherwise executes the required C<on_error> code reference.
 
+=head2 Other
+
+=head3 query
+
+    $cv = AE::cv;
+    $db->query(
+        method => 'GET',
+        query => {
+            db => 'mydb',
+            q => 'SELECT * FROM cpu_load',
+        },
+        on_response => $cv,
+    );
+    my ($response_data, $response_headers) = $cv->recv;
+
+Executes an arbitrary query using provided in C<query> arguments.
+
+The required C<on_response> code reference is executed with the raw response
+data and headers as parameters.
+
 =head1 CAVEATS
 
 Following the optimistic nature of InfluxDB this modules does not validate any
@@ -2815,7 +2817,7 @@ Alex J. G. Burzyński <ajgb@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by Alex J. G. Burzyński <ajgb@cpan.org>.
+This software is copyright (c) 2016 by Alex J. G. Burzyński <ajgb@cpan.org>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
